//////////////////////////////////////////////////////////////////////////// 
// 
// Copyright (C) DSTC Pty Ltd (ACN 052 372 577) 1993, 1994, 1995.
// Unpublished work.  All Rights Reserved.
// 
// The software contained on this media is the property of the
// DSTC Pty Ltd.  Use of this software is strictly in accordance
// with the license agreement in the accompanying LICENSE.DOC 
// file. If your distribution of this software does not contain 
// a LICENSE.DOC file then you have no rights to use this 
// software in any manner and should contact DSTC at the address 
// below to determine an appropriate licensing arrangement.
// 
//      DSTC Pty Ltd
//      Level 7, GP South
//      University of Queensland
//      St Lucia, 4072
//      Australia
//      Tel: +61 7 3365 4310
//      Fax: +61 7 3365 4311
//      Email: jcsi@dstc.qut.edu.au
// 
// This software is being provided "AS IS" without warranty of
// any kind.  In no event shall DSTC Pty Ltd be liable for
// damage of any kind arising out of or in connection with
// the use or performance of this software.
// 
//////////////////////////////////////////////////////////////////////////// 

package javax.crypto.spec;

import java.security.spec.KeySpec;

/**
 * <p>A user-chosen password that can be used with password-based
 * encryption.
 *
 * <p>The password can be viewed as some kind of raw key material, from
 * which the encryption mechanism to which it is passed derives a
 * cryptographic key.
 *
 * <p>See PKCS#5: Password-Based Encryption Standard for more details on
 * password-based encryption.
 *
 * @see javax.crypto.SecretKeyFactory
 * @see javax.crypto.spec.PBEParameterSpec
 *
 * @version 0.98, 98/07/01
 * @author Ming Yung
 *
 */
public class PBEKeySpec implements KeySpec 
{
  private char[] password;

  /**
   * Constructor that takes a password.
   * 
   * @param password the password used for password-based encryption
   */
  public PBEKeySpec(char[] password) 
  {
    this.password = password;
  }

  /**
   * Retrieves the password.
   *
   * @return the password
   */
  public char[] getPassword()
  {
    return password;
  }
}
